<?php

defined('BASEPATH') || exit('Access Denied.');

class UserUploadModel extends CI_Model {
    private $s3;
    public function __construct() {
        parent::__construct();

        $this->s3 = $this->WebsiteModel->s3_settings();
        $this->load->driver('cache', array('adapter' => 'file'));
    }
    
    public function get_upload($slug) {
        $this->load->database();
        $cv = 'imgcache-' . strtolower($slug);

        if(!$upload = $this->cache->get($cv)) {
            $res = $this->db->limit(1)->where('slug_id', strtoupper($slug))->get('user-uploads');
            if($res->num_rows()) {
                $upload = $res->row_array();
    
                $this->cache->save($cv, $upload, 86400);
            }
        }

        if($upload['user']) {
            $res = $this->db->where('id', $upload['user'])->limit(1)->get('end-users');
            if($res->num_rows()) {
                $user = $res->row_array();
                $upload['user'] = $user;
            }
        }
        
        return $upload;
    }

    public function delete_upload($id) {
        $this->load->database();
        $upload = $this->db->select('id, filename, s3, slug_id')->get_where('user-uploads', array('id' => $id));
        if($upload->num_rows()) {
            $upload = $upload->row_array();
            if($upload['s3']) {
                $this->load->library('s3_uploader');
                $this->s3_uploader->init($this->s3['s3_access'], $this->s3['s3_secret'], $this->s3['s3_bucket'], $this->s3['s3_region']);

                $this->s3_uploader->delete($upload['filename']);
            } else {
                $path = FCPATH . 'i/' . $upload['filename'];
                if(file_exists($path))
                    unlink($path);
            }

            $cv = 'imgcache-' . strtolower($upload['slug_id']);
            $this->cache->delete($cv);
            $this->db->where('id', $id)->delete('user-uploads');

            return true;
        }

        return false;
    }

    public function create_upload($name, $file_name, $user) {
        $this->load->database();

        if($this->s3['status']) {
            $this->load->library('s3_uploader');
            $this->s3_uploader->init($this->s3['s3_access'], $this->s3['s3_secret'], $this->s3['s3_bucket'], $this->s3['s3_region']);

            $path = FCPATH . 'i/' . $file_name;
            $res = $this->s3_uploader->new($file_name, $path);
            if($res) {
                if(file_exists($path))
                    unlink($path);
                $file_name = $res;
            }
        }

        $new_upload = array(
            'id' => NULL,
            'slug_id' => $name,
            'filename' => $file_name,
            'user' => ($user) ? $user['id'] : NULL,
            's3' => $this->s3['status'],
        );

        return $this->db->insert('user-uploads', $new_upload);
    }

    
    public function total_uploads($user = null) {
        $this->load->database();
        $query = $this->db;
        if($user) $query->where('user', $user);
        return $query->get('user-uploads')->num_rows();
    }

    public function recent_uploads($num) {
        $this->load->database();
        $res = $this->db->order_by('id', 'desc')->limit($num)->get('user-uploads')->result_array();
        foreach($res as $i => $upload) {
            if($upload['user']) {
                $res[$i]['user'] = $this->db->select('id,username')->where('id', $upload['user'])->get('end-users')->row_array();
            }
        }

        return $res;
    }
    
    public function weekly_uploads() {
        $this->load->database();
        return $this->db->where('YEARWEEK(`upload_date`) = YEARWEEK(NOW())')->get('user-uploads')->num_rows();
    }

    public function get_uploads($page, $user = null) {
        $this->load->database();

        $amount = 25;

        $query = $this->db->offset(($page - 1) * $amount)->limit($amount);
        if($user) $query->where('user', $user);
        $res = $query->get('user-uploads')->result_array();
        foreach($res as $i => $upload) {
            if($upload['user']) {
                $res[$i]['user'] = $this->db->select('id,username')->where('id', $upload['user'])->get('end-users')->row_array();
            }
        }

        return $res;
    }
    
    public function upload_by_id($id) {
        $this->load->database();

        return $this->db->where('id', $id)->limit(1)->get('user-uploads')->row_array();
    }
}