(function($, Dropzone) {
    "use strict";

    $(document).ready(function() {
        const url = base_url + upload + "/new";
        const img_route = 'i/';
        const block = $('#uploader-block > .uploaderSection');
    
        /* File Uploading Event Handlers. */
    
        function dragOverBlock(e)  { block.addClass('onDrag');  } // When User Drags over The Upload Block
        function dragLeaveBlock(e) { block.removeClass('onDrag');  } // When user Drags out of the Upload Block
    
        function onFileAdd(file)   { $(file.previewElement).removeClass('d-none'); } // When a New File is Added
    
        // When an error is emitted by a file.
        function onFileError(file, message = null) {
            const preview = $(file.previewElement);
            const anchor = preview.find('.success-link');
            const progress = preview.find('.upload-progress');

            anchor.html(message ? message : lang_file_not_allowed);
            anchor.removeClass('d-none');
            progress.addClass('d-none');
        }
    
        // When an upload is finished.
        function onUploadComplete(file) {
            if(file.status == "success") {
                const response = JSON.parse(file.xhr.response);
                if(response.type == 'success') {
                    const id = response.data.id;
                    const preview = $(file.previewElement);
                    
                    const anchor = preview.find('.success-link');
                    const progress = preview.find('.upload-progress');
    
                    anchor.attr('href', base_url + img_route + id);
                    anchor.html(base_url + img_route + id);
                    anchor.removeClass('d-none');
                    progress.addClass('d-none');
                } else 
                    onFileError(file, response.errors);
            } 
        }
    
        // Dropzone Preview Template
        let previewNode = document.querySelector("#droptemplate"); previewNode.id = "";
        let previewTemplate = previewNode.parentNode.innerHTML; previewNode.parentNode.removeChild(previewNode);
    
        // Create a File Uploading Dropzone.
        const dropzone = new Dropzone(
            'div#uploader-block', 
            { 
                url: url,
                method: 'post',
                paramName: 'uploads',
                previewTemplate: previewTemplate,
                previewsContainer: "#upload-previews",
                clickable: "div#upload-clickable",
                acceptedFiles: "image/png,image/jpeg, image/gif"
            }
        );
    
        // Register Events.
        dropzone.on('dragover',  dragOverBlock    );
        dropzone.on('dragleave', dragLeaveBlock   );
        dropzone.on('addedfile', onFileAdd        );
        dropzone.on('error',     onFileError      );
        dropzone.on('complete',  onUploadComplete );
    
    })
})(jQuery, Dropzone);